#include "mbed.h"
#include <string> 
using namespace std;

#define SSID "XXXXXX" // The name of your access point
#define PASS "YYYYYY" // The password of your access point
#define BAUDRATE 9600 // The baudrate of the module (I want to test this program with an ESP8266 upgraded to 00180902 firmware version to can handle at 9600 bauds. In case that you have a less version take care that you have to change it to 115200 or 57600)
#define DST_IP "220.181.111.85" // The IP address of the destination (baidu.com)

Serial serial(SERIAL_TX, SERIAL_RX);
Serial wifi(D8, D2);
 
void wifi_start();
void wifi_connect();
void wifi_httpConnect(char* ip);
bool wifi_find(const std::string& input, unsigned int timeout);
//bool wifi_find(char* input, uint8_t length, unsigned int timeout);
 
int main()
{
    serial.baud(9600);
    serial.printf("ESP8266 WebClient\n");
    
    wifi_start();
    wifi_connect();
      
    while(1)
    { 
        wifi_httpConnect(DST_IP);

        while(wifi.readable())
        {
            char c = wifi.getc();
            serial.putc(c);
            if(c=='\r')
                serial.printf("\n");
        }
        serial.printf("====\n");
        wait(5);
    }
}

void wifi_start()
{
    string s_ready("ready");
    bool ready = false;
        
    // Open wifi communications and wait for port to open:
    wifi.baud(BAUDRATE);
    //wifi.setTimeout(5000);
    
    //test if the module is ready
    serial.printf("Starting the module...\n");
    while(ready == false)
    {
        wifi.printf("AT+RST\n");
        //wait(1);
        if(wifi_find(s_ready, 5000))
        {
            serial.printf("Module is ready\n");
            ready = true;
        }
        else
        {
            serial.printf("Module has not response\n\n");
            wait(1);
            serial.printf("Restarting...\n");
        }
    }
    wait(1);
}

void wifi_connect()
{    
    bool connected = false;
    
    serial.printf("Connecting the module...\n");
    while(connected == false)
    {
        wifi.printf("AT+CWMODE=1\n");
        string cmd("AT+CWJAP=\"");
        cmd+=SSID;
        cmd+="\",\"";
        cmd+=PASS;
        cmd+="\"";
        serial.printf("%s",cmd.c_str());
        serial.printf("\n");
        wifi.printf("%s",cmd.c_str());
        wifi.printf("\n");
        
        wait(2);
        
        if(wifi_find("OK", 5000))
        {
            serial.printf("Module is connected\n");
            connected = true;
        }
        else
        {
            serial.printf("Can not connect to the WiFi\n");
            serial.printf("Trying reconnection...\n");
        }
    }   
    wait(5);
    
    //print the ip addr
    wifi.printf("AT+CIFSR\n");
    serial.printf("ip address: ");
    while(wifi.readable())
        serial.putc(wifi.getc());
    
    //set the single connection mode
    wifi.printf("AT+CIPMUX=0\n");
}

void wifi_httpConnect(char* ip)
{
    string cmd("AT+CIPSTART=\"TCP\",\"");
    cmd += ip;
    cmd += "\",80";
    wifi.printf("%s",cmd.c_str());
    wifi.printf("\n");
    serial.printf("%s",cmd.c_str());
    serial.printf("\n");
    if(wifi_find("Error", 5000))
        return;
    cmd = "GET / HTTP/1.0\r\n\r\n";
    wifi.printf("AT+CIPSEND=\n");
    wifi.printf("%d", cmd.size()); // OJO
    wifi.printf("\n");
    if(wifi_find(">", 5000))
    {
        serial.printf(">");
    }
    else
    {
        wifi.printf("AT+CIPCLOSE\n");
        serial.printf("connection timeout\n");
        wait(1);
        return;
    }
    wifi.printf("%s",cmd.c_str());
    wifi.printf("\n");
    wait(2);
}


// Wait for specific input string until timeout runs out
bool wifi_find(const std::string& input, unsigned int timeout)
{
    //string in(input);
    Timer timer;
    unsigned long end_time;
    int current_byte = 0;
    uint8_t index = 0;
    
    timer.start();
    end_time = timer.read_ms() + timeout;
    
    while(end_time >= timer.read_ms())
    {
        if(wifi.readable())
        {serial.printf("datos disponibles\n");
            //Read one byte from serial port
            current_byte = wifi.getc();
            if(current_byte != -1)
            {serial.printf("byte != -1\n");
                //Search one character at a time
                if(current_byte == input[index])
                {
                    index++;
                    //Found the string
                    if (index == input.size())
                        return true;
                    //Restart position of character to look for
                }
                else
                {
                    index = 0;
                }
            }
        }
    }
    //Timed out
    return false;
}


/*bool wifi_find(char* input, uint8_t length, unsigned int timeout)
{
    Timer timer;
    unsigned long end_time;
    int current_byte = 0;
    uint8_t index = 0;
    
    timer.start();
    end_time = timer.read_ms() + timeout;
    
    while(end_time >= timer.read_ms())
    {
        if(wifi.readable())
        {
            //Read one byte from serial port
            current_byte = wifi.getc();
            if(current_byte != -1)
            {
                //Search one character at a time
                if(current_byte == input[index])
                {
                    index++;
                    //Found the string
                    if (index == length)
                        return true;
                    //Restart position of character to look for
                }
                else
                {
                    index = 0;
                }
            }
        }
    }
    //Timed out
    return false;
}*/